% Controle no linear
% Ex 1 - Plano de fase

% Autor: Rafael Pedrosa
% Reviso: Fabrcio Brito

clear all
close all
clc

% Parametros do grfico

% Dados utilizados na construo do vetor de tempo, utilizado na resoluo
% da equao diferencial
    tmax = 10;
    n = 300;

% Escala das setas
    escala = .5;

% Definir Numero de Pontos em Cada Eixo
npont=50;

% rea onde sero plotado os vetores
xp = [-7 3;-10 10];
x1min = xp(1,1);
x1max = xp(1,2);
x2min = xp(2,1);
x2max = xp(2,2);
    
deltx1=(x1max-x1min)/npont;
deltx2=(x2max-x2min)/npont;

compr=sqrt(deltx1^2+deltx2^2)/2;

% Seleciona o quadro grfico
figure
% Fixar Janela de Grafico
hold off;
i=0;
j=0;

% Desenhar o Campo de Vetores

% Varia de x1min a x1max com passo deltx1
for x1=x1min:deltx1:x1max 
    i=i+1;
    
    % Varia de x2min a x2max com passo deltx2
    for x2=x2min:deltx2:x2max 
        x = [x1 x2]; t=0;

        % Calcula o xponto para t = 0 e x = [x1 x2] utilizando a
        % equao de estado
        xponto = equacao(t,x);
        
        % Inclinao da trajetria: m = f2(x,u)/f1(x,u)
        % A soma de 1e-8 foi utilizada para eliminar o problema de uma
        % possvel diviso por zero
        m = xponto(2)/(xponto(1)+1e-8);
        
        % compr --> mnima distncia de clculo entre dois pontos no plano
        % x1-x2
        
        % m --> tangente --> sqrt(1+m^2) --> mdulo de secante
        % --> dx1 = compr*|cos(theta)|
        dx1 = compr/sqrt(1+m^2);
        
        j=j+1;
                
        % dx2 = compr*sen(theta)
        dx2 = m*dx1;
        
        if x2 < 0
            dx2 = -dx2;
            dx1 = -dx1;
        end
        
        dx(j,i) = dx1;
        dy(j,i) = dx2;
        hold on;
    end
    j=0;
end

% Mapa vetorial
[x,y] = meshgrid(x1min:deltx1:x1max,x2min:deltx2:x2max);

% Plota os vetores: vetores construidos de (x,y)-->(x+dx,y+dy)
% Os vetores so limitados tomando em conta o valor da escala
quiver(x,y,dx,dy,escala);

axis([x1min x1max x2min x2max]);

% Tempo
t=linspace(0,tmax,n);

R = 1;
N = 10;
M = 10;

% Obtem para a matriz de condies iniciais xi das trajetrias
% As condies iniciais foram distribuidas sobre o plano x1-x2 de forma que
% ficassem igualmente espaadas entre si.
for I = 1:N+1,
    for J = 1:M+1,
        A(R,1) = x1min+((x1max-x1min)/N)*(I-1);
        A(R,2) = x2min+((x2max-x2min)/M)*(J-1);
        R=R+1;
    end
end
xi = A;

hold on;

% Resoluo da equao diferencial utilizando o 'solver' ode45
for i = 1:((N+1)*(M+1))
    [tempo,x] = ode45('equacao',t,xi(i,:));
    plot(x(:,1),x(:,2),'r');
end

% Pontos de equilbrio do sistema
plot(0,0,'k*');
plot(-3,0,'k*');

% Eixos
xlabel('x1');
ylabel('x2');
